package cn.estsh.i3plus.core.apiservice.controller.base;

import cn.estsh.i3plus.core.api.iservice.base.ISynchronizedService;
import cn.estsh.i3plus.core.api.iservice.base.ISystemInitService;
import cn.estsh.i3plus.core.api.iservice.base.ISystemLoginStrategyService;
import cn.estsh.i3plus.core.api.iservice.busi.*;
import cn.estsh.i3plus.core.apiservice.controller.busi.PersonnelController;
import cn.estsh.i3plus.core.apiservice.controller.busi.SysUserInfoController;
import cn.estsh.i3plus.core.apiservice.util.SysUserPasswordUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.platform.common.tool.ServletRequestTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.annotation.AnnoIgnoreLog;
import cn.estsh.i3plus.pojo.base.bean.BaseConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseThreadLocal;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.common.CloudPagerModel;
import cn.estsh.i3plus.pojo.model.license.ImppLicense;
import cn.estsh.i3plus.pojo.model.platform.SysLoginModel;
import cn.estsh.i3plus.pojo.model.platform.UserDetailModel;
import cn.estsh.i3plus.pojo.model.platform.UserDetailPagerModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.i3plus.sdk.dingtalk.cn.estsh.i3plus.sdk.service.IDingTalkService;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.license.ImppLicenseTool;
import cn.estsh.impp.framework.boot.license.serviceimpl.ImppLicenseDecoder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.LocaleUtils;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import com.alibaba.fastjson.JSONObject;
import com.dingtalk.api.response.OapiV2UserGetResponse;
import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.CredentialsException;
import org.apache.shiro.authc.ExpiredCredentialsException;
import org.apache.shiro.authc.IncorrectCredentialsException;
import org.apache.shiro.authc.LockedAccountException;
import org.apache.shiro.authc.UnknownAccountException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.function.BiFunction;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author : Adair Peng
 * @CreateDate : 2018-12-19 15:31
 * @Modify:
 **/
@RestController
@Api(tags = "非权限服务")
@RequestMapping("/impp/white")
public class WhiteController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(WhiteController.class);

    @Autowired
    private PersonnelController personnelController;

    @Autowired
    private SysUserInfoController sysUserInfoController;

    @Autowired
    private ISysLocaleLanguageService sysLocaleLanguageService;

    @Autowired
    private ISysLocaleResourceService sysLocaleResourceService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private IPersonnelService personnelService;

    @Autowired
    private ISystemInitService systemInitService;

    @Autowired
    private ISysFileService sysFileService;

    @Autowired
    private ISysDictionaryService dictionaryService;

    @Autowired
    private ISynchronizedService synchronizedService;

    @Autowired
    private ISysOrganizeService sysOrganizeService;

    @Autowired
    private ISysLabelTemplateService sysLabelTemplateService;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private ISysUserPasswordService userPasswordService;

    @Autowired
    private SysUserPasswordUtil userPasswordUtil;

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    @Autowired
    private ILicenseClickService licenseClickService;

    @Resource(name = CommonConstWords.IMPP_REDIS_SESSION)
    private ImppRedis redisSession;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Autowired
    private IDingTalkService dingTalkService;

    private Cache<String, Integer> unsavedLocaleResSwitch = CacheBuilder.newBuilder()
            .maximumSize(1)
            .expireAfterWrite(1, TimeUnit.MINUTES)
            .build();

    @GetMapping(value = "/sys-locale-language/list")
    @ApiOperation(value = "查询可用语言", notes = "查询可用语言")
    public ResultBean listSysLocaleLanguage() {
        try {
            List sysLocaleLanguageList = sysLocaleLanguageService.findSysLocaleLanguageByStatus(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(sysLocaleLanguageList);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/system-info")
    @ApiOperation(value = "查询可用语言", notes = "查询可用语言")
    public ResultBean getSystemInfo() {
        try {
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(configService.findSystemInfoConfig());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/session-user")
    @ApiOperation(value = "获取用户信息", notes = "获取当前登录用户账号详细信息")
    public ResultBean getSession() {
        LOGGER.info("获取用户信息...lang:{}", AuthUtil.getSessionAttribute("languageCode"));
        try {
            SessionUser sUser = AuthUtil.getSessionUser();
            LOGGER.info("用户：{}", sUser);
            return new ResultBean(true, "", sUser);
        } catch (ImppBusiException e) {
            return new ResultBean(false, e.getErrorDetail() + "-[解决]" + e.getErrorSolution());
        }
    }

    /**
     * @return
     */
    @GetMapping(value = "/auth/logout")
    @ApiOperation(value = "登出", notes = "登出")
    public ResultBean logout() {
        LOGGER.info("用户退出...");
        try {
            AuthUtil.logout();
            return new ResultBean(true, "您已安全退出。").build();
        } catch (ImppBusiException e) {
            return new ResultBean(false, e.getErrorDetail() + "-[解决]" + e.getErrorSolution());
        }
    }

    @GetMapping(value = "/auth/set-cookie")
    @ApiOperation(value = "设置cookie", notes = "设置cookie")
    public ResultBean setCookie(String token, String imppcookie) {
        try {
            return new ResultBean(true, AuthUtil.getSessionUser().getUserName() + "设置cookie完成。").build();
        } catch (ImppBusiException e) {
            return new ResultBean(false, e.getErrorDetail() + "-[解决]" + e.getErrorSolution());
        }
    }

    @GetMapping(value = "/auth/{loginStrategy}/sso-login")
    @ApiOperation(value = "登录", notes = "登录")
    public ResultBean ssoLogin(HttpServletRequest request, HttpServletResponse response,
                               @PathVariable(name = "loginStrategy") String loginStrategy) throws NoSuchFieldException {
        String fieldMapping = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.SSO_REQUEST_MAPPING);
        if (StringUtils.isEmpty(fieldMapping)) {
            return ResultBean.fail("请在平台数据库中配置表添加字段映射配置");
        }

        SysLoginModel loginModel = assembleLoginModelByRequest(request, fieldMapping);
        loginModel.setLoginStrategy(loginStrategy);
        LOGGER.info("loginModel {}", loginModel);

        ValidatorBean.beginValid(loginModel)
                .notNull("loginStrategy", loginModel.getLoginStrategy())
                .notNull("loginName", loginModel.getLoginName());

        String ssoCacheKey = assembleSsoCacheKey(loginModel);
        Object loginCacheObject = redisSession.getObject(ssoCacheKey);
        if (loginCacheObject != null) {
            LOGGER.info("使用原会话 获取缓存不为空");
            SessionUser sessionUser = (SessionUser) redisSession.getObject(loginCacheObject);
            if (sessionUser != null) {
                LOGGER.info("使用原会话 sessionUser不为空");
                if (sessionUser.getUserCode().equals(loginModel.getLoginName()) || sessionUser.getUserName().equals(loginModel.getLoginName())) {
//                    AuthUtil.setSessionUser(sessionUser);
                    LOGGER.info("使用原会话 用户名匹配返回");
                    return new ResultBean(true, sessionUser.getToken(), sessionUser);
                }
            }
        }

        String beanName = ImppEnumUtil.AUTH_LOGIN_STRATEGY.codeOfStrategyName(loginModel.getLoginStrategy());
        ISystemLoginStrategyService loginStrategyService = (ISystemLoginStrategyService) SpringContextsUtil.getBean(beanName);

        BaseThreadLocal.setData(PlatformConstWords.IS_SSO_LOGIN, true);

        ResultBean<SessionUser> resultBean = loginByStrategy(request, loginModel, loginStrategyService.login());
        if (resultBean.isSuccess()) {
            String loginFilterKey = assembleLoginFilterKey(loginModel, resultBean);
            redisSession.putObject(ssoCacheKey, loginFilterKey);
            LOGGER.info("使用新会话 放入缓存 key:{} value:{}", ssoCacheKey, loginFilterKey);
            LOGGER.info("使用新会话 sessionUser:{}", resultBean.getResultObject());
        }
        return resultBean;
    }

    private String assembleLoginFilterKey(SysLoginModel loginModel, ResultBean<SessionUser> resultBean) {
        return CommonConstWords.SESSION_USER + ":" + loginModel.getLoginPlatform() + "_" + resultBean.getResultObject().getUserId();
    }

    private String assembleSsoCacheKey(SysLoginModel loginModel) {
        return CommonConstWords.SESSION_USER + ":" + loginModel.getLoginStrategy() + ":" + loginModel.getLoginName();
    }

    private SysLoginModel assembleLoginModelByRequest(HttpServletRequest request, String fieldMapping) {
        JSONObject mappingObject = (JSONObject) JSONObject.parse(fieldMapping);

        SysLoginModel loginModel = new SysLoginModel();
        getJsonObjectParameterFromRequest("loginName", request, mappingObject).ifPresent(d -> loginModel.setLoginName(d.toString()));
        getJsonObjectParameterFromRequest("loginPwd", request, mappingObject).ifPresent(d -> loginModel.setLoginPwd(d.toString()));
        getJsonObjectParameterFromRequest("appId", request, mappingObject).ifPresent(d -> loginModel.setAppId(d.toString()));
        getJsonObjectParameterFromRequest("languageCode", request, mappingObject).ifPresent(d -> loginModel.setLanguageCode(d.toString()));
        getJsonObjectParameterFromRequest("deviceId", request, mappingObject).ifPresent(d -> loginModel.setDeviceId(d.toString()));

        return loginModel;
    }

    private Optional<Object> getJsonObjectParameterFromRequest(String key, HttpServletRequest request, JSONObject mappingObject) {
        Object data = request.getParameter(mappingObject.get(key) == null ? "" : mappingObject.get(key).toString());
        return Optional.ofNullable(data);
    }

    @GetMapping(value = "/auth/dingtalk/login")
    @ApiOperation(value = "登录", notes = "登录")
    public ResultBean loginByPhoneNumber(HttpServletRequest request, String dingTalkTmpAuthCode,
                                         @RequestParam(required = false) String languageCode,
                                         @RequestParam(required = false) String deviceId) {
        SysLoginModel loginModel = SysLoginModel.builder()
                .dingTalkTmpAuthCode(dingTalkTmpAuthCode)
                .languageCode(languageCode)
                .deviceId(deviceId)
                .loginStrategy(ImppEnumUtil.AUTH_LOGIN_STRATEGY.NAME.getCode()).build();

        String dingTalkUrl = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.DINGTALK_URL);
        String dingTalkAppKey = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.DINGTALK_APP_KEY);
        String dingTalkAppSecret = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.DINGTALK_APP_SECRET);
        String dingTalkWhiteList = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.DINGTALK_WHITE_LIST);

        if (StringUtils.isEmpty(dingTalkUrl) ||
                StringUtils.isEmpty(dingTalkAppKey) ||
                StringUtils.isEmpty(dingTalkAppSecret) ||
                StringUtils.isEmpty(dingTalkWhiteList)) {
            return ResultBean.fail("访问钉钉的系统参数缺失，请在平台的系统参数中进行配置");
        }

        Optional<OapiV2UserGetResponse> userInfo =
                dingTalkService.getUserInfo(dingTalkUrl, dingTalkAppKey, dingTalkAppSecret, loginModel.getDingTalkTmpAuthCode());
        if (userInfo.isPresent()) {
            String mobile = userInfo.get().getResult().getMobile();
            if (StringUtils.isEmpty(mobile)) {
                return ResultBean.fail("用户并未在钉钉中设置手机号 或 钉钉接口权限不足无法获取到手机号");
            }
            if (!Arrays.asList(dingTalkWhiteList.split(",")).contains(mobile)) {
                return ResultBean.fail("对指定手机号允许,权限不足,请联系管理员");
            }
            SysUser sysUserByPhone = userService.getSysUserByPhone(mobile);
            if (sysUserByPhone == null) {
                return ResultBean.fail("未根据用户手机在平台中找到相应记录 钉钉获取的手机号为" + mobile);
            }
            loginModel.setLoginName(sysUserByPhone.getUserLoginName());
            loginModel.setPhoneNumber(mobile);
        } else {
            return ResultBean.fail("获取钉钉用户信息失败");
        }
        String beanName = ImppEnumUtil.AUTH_LOGIN_STRATEGY.codeOfStrategyName(loginModel.getLoginStrategy());
        ISystemLoginStrategyService loginStrategyService = (ISystemLoginStrategyService) SpringContextsUtil.getBean(beanName);

        return loginByStrategy(request, loginModel, loginStrategyService.login());
    }

    @GetMapping(value = "/auth/login")
    @ApiOperation(value = "登录", notes = "登录")
    public ResultBean login(HttpServletRequest request, String loginName, String loginPwd,
                            @RequestParam(required = false) String languageCode,
                            @RequestParam(required = false) String deviceId) {
        SysLoginModel loginModel = SysLoginModel.builder()
                .loginName(loginName)
                .loginPwd(loginPwd)
                .languageCode(languageCode)
                .deviceId(deviceId)
                .loginStrategy(ImppEnumUtil.AUTH_LOGIN_STRATEGY.ACCOUNT.getCode()).build();
        String beanName = ImppEnumUtil.AUTH_LOGIN_STRATEGY.codeOfStrategyName(loginModel.getLoginStrategy());
        ISystemLoginStrategyService loginStrategyService = (ISystemLoginStrategyService) SpringContextsUtil.getBean(beanName);

        return loginByStrategy(request, loginModel, loginStrategyService.login());
    }


    private ResultBean loginByStrategy(HttpServletRequest request, SysLoginModel loginModel, BiFunction<HttpServletRequest, SysLoginModel, ResultBean> loginStrategy) {
        long startTime = System.currentTimeMillis();
        licenseClickService.checkLicenseNumberLogin();

        LOGGER.info("用户登陆 loginName:{} loginPwd:{} languageCode:{}", loginModel.getLoginName(), loginModel.getLoginPwd(), loginModel.getLanguageCode());
        ResultBean result = null;
        CommonEnumUtil.USER_LOGIN_STATUS userLoginStatus = CommonEnumUtil.USER_LOGIN_STATUS.LOGIN_SUCCESS;

        try {
            result = loginStrategy.apply(request, loginModel);
        } catch (IncorrectCredentialsException e) {
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_USER_PASSWORD);
            userPasswordUtil.checkLoginErrorNumber(loginModel.getLoginName(), result, e);
            userLoginStatus = CommonEnumUtil.USER_LOGIN_STATUS.WRONG_PASSWORD;
        } catch (ExpiredCredentialsException e) {
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_PASSWORD_EXPIRED);
            userLoginStatus = CommonEnumUtil.USER_LOGIN_STATUS.WRONG_USERNAME_OR_PASSWORD;
        } catch (CredentialsException e) {
            // 用户名或密码错误
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_USER_NAME);
            userLoginStatus = CommonEnumUtil.USER_LOGIN_STATUS.WRONG_USERNAME_OR_PASSWORD;
        } catch (LockedAccountException e) {
            // 账号已锁定
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_USER_LOCKING);
            userLoginStatus = CommonEnumUtil.USER_LOGIN_STATUS.USER_LOGIN_LOCKING;
        } catch (UnknownAccountException e) {
            // 用户信息不存在
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_USER_INFO_NULL);
            userLoginStatus = CommonEnumUtil.USER_LOGIN_STATUS.USER_INFO_NULL;
        } catch (AuthenticationException e) {
            // 系统异常
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_USER_PASSWORD.getDescription()).setCode(ImppExceptionEnum.LOGIN_EXCEPTION_USER_PASSWORD.getCode());
            result.setErrorMsg(e.getMessage());
            userLoginStatus = CommonEnumUtil.USER_LOGIN_STATUS.SYSTEM_ERROR;
            LOGGER.warn("Impp业务异常AuthenticationException 登录失败 ", e);
        } catch (ImppBusiException e) {
            result = ResultBean.fail(e);
            userLoginStatus = CommonEnumUtil.USER_LOGIN_STATUS.SYSTEM_ERROR;
            LOGGER.warn("Impp业务异常ImppBusiException 登录失败 ", e);
        } catch (Exception e) {
            result = ResultBean.fail(e.getMessage()).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode())
                    .setErrorMsg(LocaleUtils.getEnumLocaleResDesc(ImppExceptionEnum.SYSTEM_EXCEPTION, ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription()));
            userLoginStatus = CommonEnumUtil.USER_LOGIN_STATUS.SYSTEM_ERROR;
            LOGGER.warn("Impp业务异常Exception 登录失败 ", e);
        } finally {
            // 记录登录记录
            recordSysUserLog(null, loginModel.getLoginName(), userLoginStatus.getValue());
        }

        LOGGER.info("会员{}登陆登录完成 ,语言：{} , 登录耗时:{}", loginModel.getLoginName(), BaseThreadLocal.getThreadLanguageCode(), (System.currentTimeMillis() - startTime));
        return result;
    }

    private void recordSysUserLog(Long userId, String username, Integer userLoginStatus) {
        SysLogUserLogin log = new SysLogUserLogin();

        log.setUserId(userId);
        log.setLogLoginName(username);
        log.setLogLoginStatus(userLoginStatus);
        log.setLogLoginBrowser(ServletRequestTool.getRequestBrowser(request));
        log.setLogLoginHost(ServletRequestTool.getRequestHost(request));
        log.setLogLoginPlatform(CommonEnumUtil.LOG_LOGIN_PLATFORM.descOf(ServletRequestTool.getRequestOs(request)));
        log.setLogLoginDateTime(TimeTool.getNowTime(true));

        userService.insertSysLogUserLogin(log);
    }

    @GetMapping(value = "/find-language-code")
    @ApiOperation(value = "根据语言代码查询资源")
    public ResultBean findSysLocaleResourceByLanguageCode(String languageCode) {
        try {
            startMultiService();

            ValidatorBean.checkNotNull(languageCode, "语言代码不能为空");
            if (sysLocaleLanguageService.getSysLocaleLanguageByCode(languageCode) == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("语言不存在")
                        .build();
            }

            // 从缓存获取语言信息
            Map<Object, Object> resourceMap =
                    redisRes.getHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB + ":" + languageCode);
            if (resourceMap == null || resourceMap.size() == 0) {
                systemInitService.putAndLoadSysLocaleLanguage();
                resourceMap = redisRes.getHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB + ":" + languageCode);
            }

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultMap(resourceMap);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/sys-locale-resource/query")
    @ApiOperation(value = "按条件查询资源")
    public ResultBean findSysLocaleResource(@RequestBody CloudPagerModel<SysLocaleResource> pagerModel) {
        try {
            ListPager<SysLocaleResource> resourceList = sysLocaleResourceService.querySysLocaleResourceByPager(
                    pagerModel.getObj(), pagerModel.getPager()
            );
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setListPager(resourceList);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/find-resource")
    @ApiOperation(value = "按条件查询资源")
    public ResultBean<SysLocaleResource> findSysLocaleResource(@RequestBody SysLocaleResource resource) {
        try {
            startMultiService();

            List<SysLocaleResource> resourceList = sysLocaleResourceService.findSysLocaleResource(resource);

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(resourceList);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @GetMapping(value = "/find-language-code-soft-type")
    @ApiOperation(value = "根据语言代码查询资源")
    public ResultBean findSysLocaleResourceByLanguageCode(String languageCode, Integer softType,
                                                          @RequestParam(defaultValue = "2") Integer needPlatformRes) {
        try {
            startMultiService();

            ValidatorBean.checkNotNull(languageCode, "语言代码不能为空");
            ValidatorBean.checkNotNull(softType, "软件类型不能为空");
            if (sysLocaleLanguageService.getSysLocaleLanguageByCode(languageCode) == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("语言不存在")
                        .build();
            }

            // 从缓存获取语言信息
            Map<Object, Object> resourceMap =
                    redisRes.getHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB + ":" + languageCode + ":" + softType);
            if (resourceMap == null || resourceMap.size() == 0) {
                systemInitService.putAndLoadSysLocaleLanguage();
                resourceMap = redisRes.getHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB + ":" + languageCode + ":" + softType);
            }

            if (needPlatformRes.intValue() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) {
                Map<Object, Object> platformResMap =
                        redisRes.getHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB + ":" + languageCode + ":" + CommonEnumUtil.SOFT_TYPE.IMPP.getValue());
                ;
                resourceMap.putAll(platformResMap);
            }

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultMap(resourceMap);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @PostMapping(value = "/sys-file/insert")
    @ApiOperation(value = "新增文件资源", notes = "新增文件资源")
    public ResultBean insertSysFile(@RequestBody SysFile sysFile) {
        try {
            startMultiService();

            // 条件验证
            ValidatorBean.beginValid(sysFile)
                    .notNull("dfsFileName", sysFile.getDfsFileName())
                    .notNull("dfsGroupName", sysFile.getDfsGroupName())
                    .notNull("fileSoftType", sysFile.getFileSoftType())
                    .notNull("fileOriginalName", sysFile.getFileOriginalName());

            // 文件类型验证
            SysDictionary dictionary = dictionaryService.getSysDictionaryByParentCodeAndValue(BaseConstWords.DICTIONARY_FILE_TYPE,
                    StringTool.getStringFileSuffix(sysFile.getFileOriginalName(), true));
            if (dictionary == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_FORMAT.getCode())
                        .setErrorDetail("文件格式不支持 %s", sysFile.getFileOriginalName())
                        .setErrorSolution("请到字典管理中添加文件类型")
                        .build();
            }

            // 初始化数据
            sysFile.setFileTypeId(dictionary.getId());
            sysFile.setFileTypeName(dictionary.getName());

            ConvertBean.serviceModelInitialize(sysFile, "白名单");

            sysFile = sysFileService.insertSysFile(sysFile);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(sysFile);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据文件代码查询文件信息
     *
     * @param fileCode 文件代码
     * @return 处理结果
     */
    @GetMapping("/sys-file/get-file-code/{code}")
    @ApiOperation(value = "根据文件代码查询文件信息")
    public ResultBean getSysFileByFileCode(@PathVariable("code") String fileCode) {
        try {
            // 条件验证
            ValidatorBean.checkNotNull(fileCode, "文件代码不能为空");
            SysFile sysFile = sysFileService.getSysFileByFileCode(fileCode);

            return ResultBean.success().setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(sysFile);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @GetMapping(value = "/sys-order-no-rule/get-order-no/{code}")
    @ApiOperation(value = "根据单号规则代码，生成单号")
    public ResultBean getOrderNo(@PathVariable("code") String code) {
        try {
            ValidatorBean.checkNotNull(code, "code不能为空");

            SysOrderNoRule sysOrderNoRule = synchronizedService.nextOrderNo(code);
            return ResultBean.success("操作成功").setResultObject(sysOrderNoRule).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据单号规则代码，生成单号
     *
     * @param code 单号规则代码
     * @param num  生成数量
     * @return 处理结果
     */
    @GetMapping(value = "/sys-order-no-rule/get-order-no/{code}/{num}")
    @ApiOperation(value = "根据单号规则代码，生成单号")
    public ResultBean<String> getOrderNo(@PathVariable("code") String code, @PathVariable("num") Integer num) {
        try {
            ValidatorBean.checkNotNull(code, "code不能为空");
            ValidatorBean.checkNotZero(num, "次数不能为零");

            List<String> orderNoList = synchronizedService.nextOrderNo(code, num).stream().map(SysOrderNoRule::getOrderNo).collect(Collectors.toList());
            return ResultBean.success("操作成功").setResultList(orderNoList).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/sys-find-user/list")
    @ApiOperation(value = "查询用户列表信息", notes = "查询用户列表信息")
    public ResultBean findSysUserInfoList(UserDetailModel model) {
        try {
            List<SysUser> userList = personnelService.querySysUserByUserDetailModelOrIds(model, new ArrayList<>());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(userList);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * @return 处理结果
     */
    @GetMapping("/sys-login-platform")
    @ApiOperation(value = "登录标识", notes = "登录标识")
    public ResultBean getSysloginPlatforml() {
        return new ResultBean(true, "操作成功",
                Arrays.asList(CommonEnumUtil.LOG_LOGIN_PLATFORM.values()));
    }


    @GetMapping("/sys-organize/list")
    @ApiOperation(value = "获取全部组织", notes = "获取全部组织")
    public ResultBean<SysOrganize> findSysOrganizeList() {
        try {
            List<SysOrganize> sysOrganizeList = sysOrganizeService.findAll();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(sysOrganizeList);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询全部打印模板
     *
     * @return 处理结果
     */
    @PostMapping(value = "/sys-label-template/list")
    @ApiOperation(value = "查询全部打印模板", notes = "查询全部打印模板")
    public ResultBean<SysLabelTemplate> findSysConfigAll(@RequestBody SysLabelTemplate template) {
        try {
            List<SysLabelTemplate> sysLabelTemplateList = sysLabelTemplateService.listSysLabelTemplateDetail(template);
            ;
            return ResultBean.success("操作成功")
                    .setResultList(sysLabelTemplateList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询未翻译的资源
     *
     * @return 处理结果
     */
    @PostMapping(value = "/locale-resource/no-resource")
    @ApiOperation(value = "查询未翻译的资源", notes = "查询未翻译的资源")
    public ResultBean<SysLabelTemplate> findSysConfigAll() {
        try {
            Set<String> keys = redisRes.getKeysSet(CommonConstWords.REDIS_PREFIX_CACHE_NO_RES + "*");
            List<String> list = new ArrayList<>();
            for (String key : keys) {
                list.add(redisRes.getObject(key).toString());
            }
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据条件分页查询用户信息
     *
     * @param model 查询条件
     * @return 处理结果
     */
    @PostMapping(value = "/personnel/pager/get-user-list")
    @ApiOperation(value = "根据条件分页查询账号信息", notes = "根据条件分页查询账号信息")
    public ResultBean queryPagerByUser(@RequestBody UserDetailPagerModel model) {
        try {
            ListPager<SysUser> listPager = personnelService.querySysUser(model.getUserDetailModel(), model.getPager());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setListPager(listPager);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    /**
     * 缺失资源
     *
     * @param key
     * @param menuCode
     * @return
     */
    @AnnoIgnoreLog
    @PostMapping(value = "/miss-resource/get")
    @ApiOperation(value = "缺失资源", notes = "缺失资源")
    public ResultBean getMissResourceByKey(String key, String menuCode) {
        try {
            Integer switcher = unsavedLocaleResSwitch.asMap().get(PlatformConstWords.WEB_MISS_RESOURCE_LISTENER_SWITCH);

            if (switcher == null) {
                switcher = RedisCacheTool.getSysConfigIntVal(PlatformConstWords.WEB_MISS_RESOURCE_LISTENER_SWITCH,
                        CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
                unsavedLocaleResSwitch.put(PlatformConstWords.WEB_MISS_RESOURCE_LISTENER_SWITCH, switcher);
            }
            if (switcher == -1 || switcher == CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()) {
                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            }

            Map<String, String> missResource = new HashMap<>();
            missResource.put(key, menuCode);
            rabbitTemplate.convertAndSend(PlatformConstWords.QUEUE_IMPP_RESOURCE, JsonUtilTool.encode(missResource));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    /**
     * 缺失资源开关
     *
     * @return
     */
    @AnnoIgnoreLog
    @GetMapping(value = "/miss-resource/web-listener-switch")
    @ApiOperation(value = "缺失资源开关", notes = "缺失资源开关")
    public ResultBean getWebMissResourceSwitch() {
        try {
            int webListener = RedisCacheTool.getSysConfigIntVal(PlatformConstWords.WEB_MISS_RESOURCE_LISTENER_SWITCH,
                    CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
            return ResultBean.success("获取成功")
                    .setResultObject(webListener == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping(value = "/update-expired-password")
    @ApiOperation(value = "修改密码", notes = "松下修改当前登录用户密码")
    public ResultBean updateSysUserPassword(String loginName, String password, String newPwd) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(password, "旧密码不能为空");
            ValidatorBean.checkNotNull(newPwd, "新密码不能为空");
            ValidatorBean.checkNotNull(loginName, "用户名不能为空");

            userPasswordService.updatePasswordWizoutSession(loginName, password, newPwd.trim());
            return ResultBean.success("修改密码成功，请重新登陆").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/license")
    @ApiOperation(value = "更新授权", notes = "更新授权")
    public ResultBean updateLicense(String content) {
        try {
            ImppLicense license = ImppLicenseDecoder.getLicenseDecoder(content);
            if (Objects.nonNull(license)) {
                ImppLicenseTool.updateImppLicense(license);
            } else {
                return ResultBean.fail("错误的授权信息");
            }
            return ResultBean.success("操作成功");
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/forced-offline/{loginName}")
    @ApiOperation(value = "强制下线", notes = "强制下线")
    public ResultBean forcedOffline(@PathVariable("loginName") String loginName) {
        try {
            SysUser sysUser = userService.getSysUserByLoginName(loginName);
            ValidatorBean.checkNotNull(sysUser, "用户不存在，请检查【登录名】");
            redisSession.deleteBatchKey(CommonConstWords.SESSION_USER + ":*_" + sysUser.getId());
            return ResultBean.success("操作成功");
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/send-test-mq")
    @ApiOperation(value = "测试发送mq", notes = "测试发送mq")
    public void sendTestMq(String paramStr) {
        rabbitTemplate.convertAndSend(PlatformConstWords.QUEUE_IMPP_DEMO, paramStr);
    }

    /********************************************SWEB所需接口*******************************************/
    /**
     * 添加用户信息
     *
     * @param model 用户信息
     * @return 处理结果
     */
    @PostMapping(value = "/user/insert")
    @ApiOperation(value = "添加用户信息", notes = "返回内容添加用户信息")
    public ResultBean insertUserDetailModel(@RequestBody UserDetailModel model) {
        return personnelController.insertUserDetailModel(model);
    }

    /**
     * 修改用户信息
     *
     * @param model 用户信息
     * @return 处理结果
     */
    @PutMapping(value = "/user/update")
    @ApiOperation(value = "修改用户信息", notes = "返回内容添加用户信息")
    public ResultBean updateUserDetailModel(@RequestBody UserDetailModel model) {
        return personnelController.updateUserDetailModel(model);
    }

    /**
     * 删除用户信息
     *
     * @param id 用户id
     * @return 处理结果
     */
    @DeleteMapping(value = "/user/delete/{id}")
    @ApiOperation(value = "删除用户信息", notes = "根据用户信息id删除信息")
    public ResultBean deleteSysUserInfo(@PathVariable("id") String id) {
        return personnelController.deleteSysUserInfo(id);
    }

    /**
     * 查询用户信息
     *
     * @param cloudPagerModel 查询条件
     * @return 处理结果
     */
    @PostMapping(value = "/user/query")
    @ApiOperation(value = "查询用户信息", notes = "复杂查询用户信息带分页")
    public ResultBean querySysUserInfo(@RequestBody CloudPagerModel<SysUserInfo> cloudPagerModel) {
        return sysUserInfoController.querySysUserInfo(cloudPagerModel.getObj(), cloudPagerModel.getPager());
    }
}
