package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ICoreMemTreeService;
import cn.estsh.i3plus.core.api.iservice.busi.IPersonnelService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysMessageService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysRoleService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserInfoService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserPasswordService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.EncryptTool;
import cn.estsh.i3plus.platform.common.tool.ExcelTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.FileContentTypeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.AccountExportModel;
import cn.estsh.i3plus.pojo.model.platform.UserExportModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.sql.ResultSet;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toCollection;

/**
 * @Description : 系统用户操作服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-25 21:46
 * @Modify :
 **/
@RestController
@Api(tags = "系统用户服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/sys-user")
public class SysUserController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysUserController.class);

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private ISysUserInfoService sysUserInfoService;

    @Autowired
    private ISysMessageService sysMessageService;

    @Autowired
    private IPersonnelService personnelService;

    @Autowired
    private ICoreMemTreeService coreMemTreeService;

    @Autowired
    private ISysUserPasswordService userPasswordService;

    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private EntityManager entityManager;

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    /**
     * 新增系统用户
     *
     * @param sysUser 用户
     * @param roleIds 角色id
     * @return 处理结果
     */
    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增用户", notes = "新增系统用户")
    public ResultBean insertSysUser(SysUser sysUser, String[] roleIds) {
        try {
            startMultiService();

            ConvertBean.modelSafeArrayNumber(roleIds, true);

            ValidatorBean.beginValid(sysUser)
                    .numberCheck("userInfoId", sysUser.getUserInfoId())
                    .notNull("userLoginName", sysUser.getUserLoginName())
                    // FIXME 松下取消邮箱校验
//                            .notNull("userEmail",sysUser.getUserEmail())
                    .notNull("userLoginPassword", sysUser.getUserLoginPassword());

            if (roleIds == null || roleIds.length <= 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("角色信息不能为空")
                        .build();
            }

            // 用户唯一校验
            personnelService.checkSysUserOnly(sysUser);

            SysUserInfo userInfo = sysUserInfoService.getSysUserInfoById(sysUser.getUserInfoId());
            if (userInfo == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                        .setErrorDetail("用户信息不存在请重新操作")
                        .build();
            }

            //新增初始化
            sysUser.setUserName(userInfo.getName());
            sysUser.setUserStatus(CommonEnumUtil.USER_STATUS.ENABLE.getValue());
            sysUser.setIsValid(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            sysUser.setUserType(CommonEnumUtil.USER_TYPE.USER.getValue());
            sysUser.setUserLoginNum(0);
            ConvertBean.modelInitialize(sysUser, AuthUtil.getSessionUser());

            SysUser user = sysUserService.insertSysUser(sysUser);

            // 更新用户关系
            personnelService.refreshRefSysUserRole(user.getId(), StringTool.getArrayLong(roleIds));

            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(user);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改用户
     *
     * @param sysUser 用户
     * @param roleIds 角色id
     * @return 处理结果
     */
    @PutMapping(value = "/update")
    @ApiOperation(value = "修改用户", notes = "修改用户信息")
    public ResultBean updateSysUser(SysUser sysUser, String[] roleIds) {
        try {
            startMultiService();

            ConvertBean.modelSafeArrayNumber(roleIds, true);
            ValidatorBean.beginValid(sysUser)
                    .numberCheck("id", sysUser.getUserInfoId())
                    .numberCheck("userInfoId", sysUser.getUserInfoId())
                    .notNull("userLoginName", sysUser.getUserLoginName());
            // FIXME 松下取消邮箱校验
//                    .notNull("userEmail",sysUser.getUserEmail());

            if (roleIds == null || roleIds.length <= 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("角色信息不能为空")
                        .build();
            }

            // 用户唯一校验
            personnelService.checkSysUserOnly(sysUser);

            // 用户信息封装
            SysUserInfo userInfo = sysUserService.getSysUserInfoById(sysUser.getUserInfoId());
            SysUser user = sysUserService.getSysUserById(sysUser.getId());
            if (userInfo == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                        .setErrorDetail("用户信息不存在请重新操作")
                        .build();
            }

            // 修改密码
            if (sysUser.getUserLoginPassword() != null && sysUser.getUserLoginPassword().trim().length() > 0) {
                String md5 = EncryptTool.hexMD5(sysUser.getUserLoginPassword());
                user.setUserLoginPassword(md5);
            }

            user.setUserInfoId(userInfo.getId());
            user.setUserName(userInfo.getName());
            user.setUserLoginName(sysUser.getUserLoginName());
            user.setUserEmail(sysUser.getUserEmail());
            user.setUserPhone(sysUser.getUserPhone());
            ConvertBean.modelUpdate(sysUser, AuthUtil.getSessionUser());

            // 更新用户关系
            personnelService.refreshRefSysUserRole(user.getId(), StringTool.getArrayLong(roleIds));
            sysUserService.updateSysUser(user);

            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(user);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改用户状态
     *
     * @param id     用户id
     * @param status 状态
     * @return 处理结果
     */
    @PutMapping(value = "/status")
    @ApiOperation(value = "修改用户状态", notes = "根据ID修改用户状态")
    public ResultBean updateStatus(String id, int status) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "用户id 不能为空");
            ValidatorBean.checkNotZero(status, "用户状态不能为空");

            sysUserService.updateSysUserStatus(Long.parseLong(id), status, AuthUtil.getSessionUser());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 批量修改系统功能状态
     *
     * @param ids    系统功能id数组
     * @param status 状态
     * @return 处理结果
     */
    @PutMapping(value = "/batch-status")
    @ApiOperation(value = "批量修改系统功能状态", notes = "根据ID修改系统功能状态")
    public ResultBean updateBatchStatus(String[] ids, int status) {
        try {
            ids = ConvertBean.modelSafeArrayNumber(ids, true);

            // 数据校验
            ValidatorBean.checkNotNull(ids, "功能id 不能为空");
            ValidatorBean.checkNotZero(status, "功能状态不能为空");

            sysUserService.updateBatchSysUserStatus(StringTool.getArrayLong(ids), status, AuthUtil.getSessionUser());

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改当前登录用户密码
     *
     * @param password 原密码
     * @param newPwd   新密码
     * @return 处理结果
     */
    @PutMapping(value = "/update-password")
    @ApiOperation(value = "修改密码", notes = "修改当前登录用户密码")
    public ResultBean updateSysUserPassword(String password, String newPwd) {
        try {
            SessionUser sessionUser = AuthUtil.getSessionUser();
            // 数据校验
            ValidatorBean.checkNotNull(password, "旧密码不能为空");
            ValidatorBean.checkNotNull(newPwd, "新密码不能为空");

            userPasswordService.updatePassword(sessionUser.getUser().getId(), password, newPwd.trim());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询所有用户
     *
     * @return 查询所有用户
     */
    @GetMapping(value = "/list")
    @ApiOperation(value = "查询所有用户", notes = "查询所有系统用户")
    public ResultBean findAll() {
        try {
            List<SysUser> list = sysUserService.listByUserOrganizeCode();
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/export/ref-user-role")
    @ApiOperation(value = "导出用户角色关系", notes = "导出用户角色关系")
    public ResultBean exportUserRoleRef(HttpServletResponse response) {
        File file = null;
        try {
            List<SysRefUserRole> list = sysUserService.findValidUserRoleRef();
            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            String fileName = "user-role-ref-" + System.currentTimeMillis() + ".xls";
            file = new File(fileName);
            file.createNewFile();
            excelTool.exportData(file, list, SysRefUserRole.class
                    , (LinkedHashMap<String, String>) ExcelTool.getColName(SysRefUserRole.class));

            response.setContentType("application/force-download"); // 设置强制下载不打开
            response.addHeader("Content-Disposition", "attachment;fileName=" + fileName); // 设置文件名
            response.addHeader("Content-type", FileContentTypeTool.getContentType(StringTool.getStringFileSuffix(fileName, true)));

            // 设置文件名
            try (BufferedInputStream bis = new BufferedInputStream(new DataInputStream(new FileInputStream(file)))) {
                OutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
            } catch (Exception e) {
                LOGGER.error("用户角色关联导出异常", e);
            }
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        } finally {
            if (file != null) {
                FileUtils.deleteQuietly(file);
            }
        }
    }

    /**
     * 查询系统用户
     *
     * @param user  查询条件
     * @param pager 分页数据
     * @return 处理结果
     */
    @GetMapping(value = "/query-user-by-pager")
    @ApiOperation(value = "查询系统用户", notes = "组合查询用户，外带分页查询(isValid = 1)")
    public ResultBean querySysUserByPager(SysUser user, Pager pager) {
        try {
            List<Long> idList = new ArrayList<>();
            if (user.getOrganizeId() == null || user.getOrganizeId() <= 0) {
                if (AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)) {
                    user.setOrganizeCode(null);
                    user.setOrganizeId(null);
                } else {
                    user.setOrganizeId(AuthUtil.getOrganize().getId());
                    user.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
                }
            }

            if (user.getRoleId() != null) {
                List<SysRefUserRole> refUserRoleList = personnelService.findSysRefUserRole(user.getRoleId(), null);
                if (refUserRoleList != null && refUserRoleList.size() > 0) {
                    refUserRoleList.forEach(ref -> idList.add(ref.getUserId()));
                }
            }

            List<Long> departmentIdList = coreMemTreeService.findDepartmentIdListByOrganizeIdAndDepartmentId(user.getOrganizeId(), user.getDepartmentId());
            if (departmentIdList != null && departmentIdList.size() > 0) {
                List<SysRefUserDepartment> refList = personnelService.findSysRefUserDepartmentByDepartmentIdList(departmentIdList);
                if (refList != null && refList.size() > 0) {
                    refList.forEach(ref -> idList.add(ref.getUserId()));
                }
            }

            ListPager<SysUser> listPager = sysUserService.querySysUserByPager(user, idList, pager);
            return ResultBean.success("操作成功").setListPager(listPager).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/query-user-in-org")
    @ApiOperation(value = "根据orgCode查询系统用户")
    public ResultBean queryUserInOrg(String organizeCode) {
        try {
            List<SysUser> sysUsers = sysUserService.querySysUserByOrganizeCode(organizeCode);
            return ResultBean.success("操作成功").setResultList(sysUsers).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @GetMapping(value = "/role/list")
    @ApiOperation(value = "查询所有角色", notes = "查询所有系统角色")
    public ResultBean findRoleAll() {
        try {
            List<SysRole> list = roleService.findAll();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    /**
     * 删除账号
     *
     * @param id 账号id
     * @return 处理结果
     */
    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "删除账号", notes = "删除制定账号信息")
    public ResultBean delete(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "用户id 不能为空");

            sysUserService.deleteSysUserById(Long.parseLong(id));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id批量删除
     *
     * @param ids id数组
     * @return 处理结果
     */
    @DeleteMapping(value = "/batch-delete")
    @ApiOperation(value = "根据id批量删除", notes = "根据id批量删除")
    public ResultBean deleteBatchSysUserByIds(String[] ids) {
        try {
            ids = ConvertBean.modelSafeArrayNumber(ids, true);
            if (ids.length == 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的用户。")
                        .build();
            }

            sysUserService.deleteBatchSysUserById(StringTool.getArrayLong(ids), AuthUtil.getSessionUser());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询用户的所有角色权限关系
     *
     * @param userId 用户id
     * @return 处理结果
     */
    @GetMapping(value = "/find-ref-role/{userId}")
    @ApiOperation(value = "查询用户的所有角色权限关系", notes = "查询用户的所有角色权限关系")
    public ResultBean findRefRoleMenu(@PathVariable("userId") String userId) {
        try {
            ValidatorBean.checkNotNull(userId, "用户 id 不能为空");
            List<SysRefUserRole> list = sysUserService.findSysRefUserRoleByUserId(Long.parseLong(userId));
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id重置用户密码
     *
     * @param id 用户id
     * @return 处理结果
     */
    @PostMapping("/reset-pwd/{id}")
    @ApiOperation(value = "密码重置", notes = "根据id重置用户密码")
    public ResultBean updateResetPassword(@PathVariable String id) {
        try {
            startMultiService();

            SysUser user = sysUserService.getSysUserById(Long.parseLong(id));
            if (user != null) {
                Integer num = personnelService.checkSysUserResetPassword(user);
                num = CommonConstWords.CONFIG_PWD_RESET_PASSWORD_DEFAULT - num;


                String password = RandomStringUtils.random(6, true, false);
                userPasswordService.updatePasswordResetLastModifyTime(user.getId(), password);

                String content = "系统提示：\n" +
                        "\t" + getSessionUser().getUserName() + "使用密码重置功能帮您重置了【" + user.getUserLoginName() + "】账号的密码，" +
                        "新密码是：" + password + "";

                SysMessage message = new SysMessage();
                message.setMessageTitle("重置密码");
                message.setMessageContent(content);
                message.setMessageType(ImppEnumUtil.MESSAGE_TYPE.MAIL.getValue());
                message.setMessageContentType(ImppEnumUtil.MESSAGE_TYPE_CONTENT.TEXT.getValue());
                message.setMessageSenderId(getSessionUser().getUser().getId());
                message.setMessageSenderNameRdd(getSessionUser().getUserName());
                message.setMessageReceiversId(user.getId().toString());
                message.setMessageReceiversNameRdd(user.getUserName());
                message.setMessageReceiverType(ImppEnumUtil.MESSAGE_RECEIVER_TYPE.INTERNAL.getValue());

                sysMessageService.doSendSysMessage(message);

                LOGGER.info("系统提示：{} 使用密码重置功能帮{}重置了{}账号的密码，新密码是：{}",
                        getSessionUser().getUserName(), user.getUserName(), user.getUserLoginName(), password);

                String result = "操作成功，【" + user.getUserName() + "】账号今日重置密码次数剩余" + num + "次";
                return ResultBean.success(result).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            } else {
                return ResultBean.success("操作失败")
                        .setCode(ResourceEnumUtil.MESSAGE.FAIL.getCode())
                        .setErrorMsg("用户不存在");
            }
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id重置用户密码,自己设置密码
     *
     * @param id 用户id
     * @return 处理结果
     */
    @PostMapping("/reset-pwd/{id}/{password}")
    @ApiOperation(value = "密码重置(输入)", notes = "根据id和录入密码重置用户密码")
    public ResultBean updateResetPassword(@PathVariable("id") String id, @PathVariable("password") String password) {
        try {
            startMultiService();

            SysUser user = sysUserService.getSysUserById(Long.parseLong(id));
            if (user != null) {
                Integer num = personnelService.checkSysUserResetPassword(user);
                num = CommonConstWords.CONFIG_PWD_RESET_PASSWORD_DEFAULT - num;

                userPasswordService.updatePasswordResetLastModifyTime(user.getId(), password);
                LOGGER.info("系统提示：{} 使用密码重置功能帮{}重置了{}账号的密码，新密码是：{}",
                        getSessionUser().getUserName(), user.getUserName(), user.getUserLoginName(), password);

                String result = "操作成功，【" + user.getUserName() + "】账号今日重置密码次数剩余" + num + "次";
                return ResultBean.success(result).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            } else {
                return ResultBean.success("操作失败")
                        .setCode(ResourceEnumUtil.MESSAGE.FAIL.getCode())
                        .setErrorMsg("用户不存在");
            }
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 发送验证码
     *
     * @param email 邮箱
     * @return 处理结果
     */
    @GetMapping("/verification/get-email")
    @ApiOperation(value = "发送验证码", notes = "发送邮件验证码")
    public ResultBean getEmailVerification(String email) {
        try {
            SysUser user = sysUserService.getSysUserByEmail(email);
            if (user != null) {
                String verification = RandomStringUtils.random(6, true, false);

                String redisKey = CommonConstWords.SESSION_VERIFICATION_USER_EMAIL + "_" + getSessionUser().getUserId();
                redisCore.putObject(redisKey, verification + "-" + email, 180);

                String content = "系统提示：\n" +
                        "\t密码找回验证码：【" + verification + "】";

                SysMessage message = new SysMessage();
                message.setMessageTitle("验证码");
                message.setMessageContent(content);
                message.setMessageType(ImppEnumUtil.MESSAGE_TYPE.MAIL.getValue());
                message.setMessageContentType(ImppEnumUtil.MESSAGE_TYPE_CONTENT.TEXT.getValue());
                message.setMessageSenderId(getSessionUser().getUser().getId());
                message.setMessageSenderNameRdd(getSessionUser().getUserName());
                message.setMessageReceiversId(user.getId().toString());
                message.setMessageReceiversNameRdd(user.getUserName());
                message.setMessageSendTime(TimeTool.getNowTime(true));
                message.setMessageReceiverType(ImppEnumUtil.MESSAGE_RECEIVER_TYPE.INTERNAL.getValue());

                sysMessageService.doSendSysMessage(message);

                LOGGER.info("系统提示：\t 密码找回验证码【{}】", verification);
                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            } else {
                return ResultBean.success("操作失败")
                        .setCode(ResourceEnumUtil.MESSAGE.FAIL.getCode())
                        .setErrorMsg("用户不存在");
            }
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
// TODO 密码待修复
//    /**
//     * 通过邮箱的验证码修改密码
//     * @param password 新密码
//     * @param email 邮箱
//     * @param verification 验证码不能为空
//     * @return 处理结果
//     */
//    @PostMapping("/password/update-verification")
//    @ApiOperation(value = "修改密码",notes = "通过邮箱的验证码修改密码")
//    public ResultBean getEmailVerification(String password,String email,String verification){
//        try {
//            ValidatorBean.checkNotNull(verification,"验证码不能为空");
//            ValidatorBean.checkNotNull(password,"新密码不能为空");
//            ValidatorBean.checkNotNull(email,"邮件不能为空");
//
//            personnelService.checkSysUserPassword(password);
//
//            String redisKey = CommonConstWords.SESSION_VERIFICATION_USER_EMAIL + "_" + getSessionUser().getUserId();
//            Object redisValue = redisCore.getObject(redisKey);
//            if(redisValue != null){
//                if(redisValue.toString().indexOf(verification) >= 0 ){
//                    if(redisValue.toString().indexOf(email) >= 0 ){
//                        SysUser user = sysUserService.getSysUserByEmail(email);
//                        user.setUserLoginPassword(EncryptTool.hexMD5(password));
//                        sysUserService.updateSysUser(user);
//                        return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
//                    }else {
//                        return ResultBean.fail("操作失败")
//                                .setCode(ResourceEnumUtil.MESSAGE.FAIL.getCode())
//                                .setErrorMsg("请勿修改邮箱信息");
//                    }
//                }else {
//                    return ResultBean.fail("操作失败")
//                            .setCode(ResourceEnumUtil.MESSAGE.FAIL.getCode())
//                            .setErrorMsg("验证码错误请重新输入");
//                }
//            }else {
//                return ResultBean.fail("操作失败")
//                        .setCode(ResourceEnumUtil.MESSAGE.FAIL.getCode())
//                        .setErrorMsg("验证码已过期");
//            }
//        } catch (ImppBusiException busExcep) {
//            return ResultBean.fail(busExcep);
//        } catch (Exception e) {
//            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
//        }
//    }

    /**
     * 修改用户操作组织id
     *
     * @param idStr         用户
     * @param organizeIdStr 组织id
     * @return 处理结果
     */
    @PutMapping("/update-organize")
    @ApiOperation(value = "修改用户操作组织id")
    public ResultBean updateSysUserOrganize(String idStr, String organizeIdStr) {
        try {
            startMultiService();

            ValidatorBean.checkNotNull(idStr, "id不能为空");
            SysUser sysUser = sysUserService.getSysUserById(Long.parseLong(idStr));
            ValidatorBean.checkNotNull(sysUser, "用户不存在");

            sysUser.setOrganizeId(Long.parseLong(organizeIdStr));

            sysUserService.updateSysUser(sysUser);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @GetMapping("/export")
    @ApiOperation(value = "导出账号信息", notes = "导出账号信息")
    public void exportSysUserInfo(SysUser sysUser, Pager pager, HttpServletResponse response) throws Exception {
        File file = null;
        if (sysUser == null) {
            sysUser = new SysUser();
        }
        try {

            //设置默认值
//            sysUser.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
//            sysUser.setIsValid(CommonEnumUtil.IS_VAILD.VAILD.getValue());
//            sysUser.setIsDeleted(CommonEnumUtil.IS_DEAL.NO.getValue());
            ResultBean resultBean = querySysUserByPager(sysUser, pager);
            List<SysUser> list = resultBean.resultList;
//            if (!resultBean.isSuccess() || ObjectUtils.isEmpty(resultBean.resultList)) {
//
//            }
            List<AccountExportModel> exportModelList = new ArrayList<>();


            List<SysRefUserRole> sysRefUserRoleList;
            StringBuilder roleName;
            for (SysUser user : list) {
                exportModelList.add(new AccountExportModel(
                        user.getUserName(),
                        user.getUserLoginName(),
                        user.getUserEmail(),
                        user.getUserPhone(),
                        user.getDepartmentNameRdd(),
                        user.getUserLoginNum(),
                        user.getUserLoginLastDateTime(),
                        user.getUserStatus())
                );

            }

            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            String fileName = "user-" + System.currentTimeMillis() + ".xls";
            file = new File(fileName);
            file.createNewFile();
            File excle = excelTool.exportData(file, exportModelList, AccountExportModel.class
                    , (LinkedHashMap<String, String>) ExcelTool.getColName(AccountExportModel.class));

            response.setContentType("application/force-download"); // 设置强制下载不打开
            response.addHeader("Content-Disposition", "attachment;fileName=" + fileName); // 设置文件名
            response.addHeader("Content-type", FileContentTypeTool.getContentType(StringTool.getStringFileSuffix(fileName, true)));

            // 设置文件名
            try (BufferedInputStream bis = new BufferedInputStream(new DataInputStream(new FileInputStream(excle)))) {
                OutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
            } catch (Exception e) {
                LOGGER.error("账号导出异常", e);
            }
        } finally {
            FileUtils.deleteQuietly(file);
        }
    }


    @GetMapping("/find-ref-role-menu/{userId}")
    @ApiOperation(value = "查询人员的角色和权限信息", notes = "查询人员的角色和权限信息")
    public ResultBean<SysRefRoleMenu> findUserRefRoleMenu(@PathVariable("userId") String userId) {
        try {
            ValidatorBean.checkNotNull(userId, "用户 id 不能为空");
            List<SysRefRoleMenu> resultList = sysUserService.findSysRefUserRoleMenuByUserId(Long.parseLong(userId));
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(resultList);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
