package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.IPersonnelService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysMenuService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysRoleService;
import cn.estsh.i3plus.core.apiservice.dao.IUserPermissionDao;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.jpa.dao.BaseRepository;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.tool.HqlPack;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysMenu;
import cn.estsh.i3plus.pojo.platform.bean.SysRefRoleMenu;
import cn.estsh.i3plus.pojo.platform.bean.SysRole;
import cn.estsh.i3plus.pojo.platform.bean.SysUser;
import cn.estsh.i3plus.pojo.platform.repository.SysRefRoleMenuRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysRefUserRoleRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysRoleRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.i3plus.pojo.wms.bean.WmsPart;
import cn.estsh.impp.framework.base.service.CrudService;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections4.ListUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static cn.estsh.i3plus.platform.common.util.PlatformConstWords.IMPP_REQUEST_ID;
import static cn.estsh.i3plus.platform.common.util.PlatformConstWords.SQL_SERVER_PARAM_MAX_MUM;

/**
 * @Description : 系统角色业务接口实现
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-22 16:58:43.779
 * @Modify :
 **/
@Service
public class SysRoleService extends CrudService<SysRole> implements ISysRoleService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysRoleService.class);

    @Autowired
    private SysRoleRepository roleRDao;
//
//    @Autowired
//    private SysMenuRepository sysMenuRDao;

    @Autowired
    private SysRefRoleMenuRepository refRoleMenuRDao;

    @Autowired
    private IPersonnelService personnelService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private IUserPermissionDao userPermissionDao;

    @Autowired
    private SysRefUserRoleRepository refUserRoleRDao;

    @Override
    public BaseRepository<SysRole, Long> getRepository() {
        return roleRDao;
    }

    @Override
    public DdlPackBean getFindPagerPackBean(SysRole bean) {
        return CoreHqlPack.packDdlBeanSysRole(bean);
    }

    @Override
    @ApiOperation(value = "更新角色信息", notes = "根据ID 批量更新角色状态信息")
    public void updateSysRoleStatusByIds(Long[] ids, int status, SessionUser user) {
        LOGGER.info("系统功能 SYS_MENU  ids:{}  status:{} user:{}", ids, status, user);
        StringBuffer where = new StringBuffer();

        HqlPack.getInPack(StringUtils.join(ids, ","), "id", where);
        roleRDao.updateByHqlWhere(where.toString(), "roleStatus", status);
    }

    @Override
    @ApiOperation(value = "刷新角色菜单关系", notes = "刷新 角色 菜单关系信息")
    public void refreshSysRoleRef(Long roleId, Long[] ids, String userName) {
        LOGGER.info("系统角色 SYS_ROLE :{}", roleId);
        LOGGER.info("系统角色 List<SysMenu> :{}", ids);

        refreshBatchSysRoleRef(new Long[]{roleId}, ids, userName);
    }

    @Override
    @ApiOperation(value = "刷新角色菜单关系", notes = "批量刷新 角色 菜单关系信息")
    public void refreshBatchSysRoleRef(Long[] roleIds, Long[] ids, String userName) {
        LOGGER.info("系统角色 String[] :{}", roleIds);
        LOGGER.info("系统角色 List<SysMenu> :{}", ids);

        String roleWhere = CoreHqlPack.packHqlIds("id", roleIds);
        List<SysRole> roleList = roleRDao.findByHqlWhere(roleWhere);

        if (roleList != null && roleList.size() > 0) {
            List<String> moduleListRdd = new ArrayList<>();
            List<String> featuresListRdd = new ArrayList<>();           // 冗余
            List<SysRefRoleMenu> refs = new ArrayList<>();                 // 角色权限关系

            // 删除角色权限关系
            refRoleMenuRDao.deleteByPropertyIn("roleId", roleIds);

            if (ids != null && ids.length > 0) {
                List<SysMenu> list = new ArrayList<>();
                // sqlserver限制最大传参数量
                for (List<Long> idList : ListUtils.partition(Arrays.asList(ids), SQL_SERVER_PARAM_MAX_MUM)) {
                    System.out.println("切割大小：" + idList.size());
                    list.addAll(menuService.findByIdList(idList));
                }
                SysRefRoleMenu ref = null;

                // 插入角色权限关系
                if (list != null && list.size() > 0) {
                    for (SysMenu menu : list) {
                        for (SysRole role : roleList) {
                            ref = new SysRefRoleMenu();
                            ref.setRoleId(role.getId());
                            ref.setRoleNameRdd(role.getName());
                            ref.setMenuNameRdd(menu.getName());
                            ref.setMenuId(menu.getId());
                            ref.setSoftType(menu.getSoftType());
                            ref.setMenuTypeRdd(menu.getMenuType());
                            ConvertBean.serviceModelInitialize(ref, userName);

                            refRoleMenuRDao.insert(ref);
                        }

                        // 冗余信息封装
                        if (CommonEnumUtil.METHOD_LEVEL.MODULE.getValue() == menu.getMenuType().intValue()) {
                            moduleListRdd.add(menu.getName());
                        }
                        if (CommonEnumUtil.METHOD_LEVEL.METHOD.getValue() == menu.getMenuType().intValue()) {
                            featuresListRdd.add(menu.getName());
                        }
                    }
                }
            }

            // 冗余信息
            for (SysRole role : roleList) {
                role.setModuleNumber(moduleListRdd.size());
                role.setMenuNumber(featuresListRdd.size());
            }

            roleRDao.saveAll(roleList);
        } else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("角色信息不存在")
                    .setErrorSolution("请重新输入")
                    .build();
        }
    }

    @Override
    public void refreshSysRoleRef(Long roleId, Integer softType, List<Long> idList, String userName) {
        SysRole role = roleRDao.getById(roleId);
        ValidatorBean.checkNotNull(role, "不存在的角色信息");
        ValidatorBean.checkNotNull(softType, "不存在的产品信息");

        SysMenu menu = new SysMenu();
        menu.setSoftType(softType);
        List<SysMenu> menuList = menuService.findAllByBean(menu);

        refRoleMenuRDao.deleteByProperties(new String[]{"roleId", "softType"}, new Object[]{roleId, softType});
        if (CollectionUtils.isNotEmpty(idList)) {
            List<SysRefRoleMenu> refs = new ArrayList<>();
            SysRefRoleMenu ref = null;

            for (Long id : idList) {
                if (CollectionUtils.isNotEmpty(menuList)) {
                    for (SysMenu sysMenu : menuList) {
                        if (sysMenu.getId().equals(id)) {
                            ref = new SysRefRoleMenu();
                            ref.setRoleId(role.getId());
                            ref.setRoleNameRdd(role.getName());
                            ref.setSoftType(sysMenu.getSoftType());
                            ref.setMenuNameRdd(sysMenu.getName());
                            ref.setMenuId(sysMenu.getId());
                            ref.setMenuTypeRdd(sysMenu.getMenuType());
                            ConvertBean.serviceModelInitialize(ref, AuthUtil.getSessionUser());

                            refs.add(ref);
                            break;
                        }
                    }
                }
            }

            if (CollectionUtils.isNotEmpty(refs)) {
                refRoleMenuRDao.saveAll(refs);
            }
        }
    }

    @Override
    @ApiOperation(value = "查信角色菜单关系", notes = "查询 ID 查询角色菜单关系")
    public List<SysRefRoleMenu> findSysRefRoleMenuByRoleId(Long roleId) {
        LOGGER.info("系统角色权限关系 RefRoleMenu find By RoleId");
        return refRoleMenuRDao.findByProperty(
                new String[]{"roleId", "isValid"},
                new Object[]{roleId, CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()}
        );
    }

    @Override
    public List<SysRefRoleMenu> findSysRefRoleMenuByRoleIdList(List<Long> idList) {
        if (ObjectUtils.isEmpty(idList)) {
            return null;
        }
        DdlPackBean packBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getInPackList(idList, "roleId", packBean);
//        LOGGER.info("hql {}",packBean.getPackedHql());
        return refRoleMenuRDao.findByHqlWhere(packBean);
    }


    @Override
    public List<SysRefRoleMenu> findSysRefRoleMenuByRoleId(Long roleId, Integer softType) {
        LOGGER.info("系统角色权限关系 RefRoleMenu find By RoleId");
        DdlPackBean ddlPackBean = new DdlPackBean();
        DdlPreparedPack.getNumEqualPack(roleId, "roleId", ddlPackBean);
        DdlPreparedPack.getNumEqualPack(softType, "softType", ddlPackBean);
        DdlPreparedPack.getNumEqualPack(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue(), "isValid", ddlPackBean);

        return refRoleMenuRDao.findByHqlWhere(ddlPackBean);
    }

    @Override
    public List<SysRole> findSysRoleByInIdAndStatus(Long[] ids, Integer status) {
        LOGGER.info("系统角色 SYS_ROLE findSysRoleByInIdAndStatus ids:{} , status:{}", ids, status);
        StringBuffer findWhere = new StringBuffer();
        HqlPack.getInPack(StringUtils.join(ids, ","), "id", findWhere);
        HqlPack.getNumEqualPack(status, "roleStatus", findWhere);
        return roleRDao.findByHqlWhere(findWhere.toString());
    }

    @Override
    public List<SysRole> findValidSysRole() {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean();
        return roleRDao.findByHqlWhere(packBean);
    }

    @Override
    @ApiOperation(value = "查信用户的所有角色信息", notes = "查信用户的所有角色信息")
    public List<SysRole> findSysRoleByUser(SysUser user) {
        LOGGER.info("系统角色 查询用户所有角色");
        return userPermissionDao.findSysRoleByUser(user);
    }

    @Override
    public ListPager<SysRole> querySysRole(SysRole role, List<Long> idList, Pager pager) {
        DdlPackBean packBean = CoreHqlPack.packDdlBeanSysRole(role, idList);
        pager = PagerHelper.getPager(pager, roleRDao.findByHqlWhereCount(packBean));
        return new ListPager(roleRDao.findByHqlWherePage(packBean, pager), pager);
    }

    @Override
    public void checkSysRoleOnly(SysRole role) {
        if (role != null && roleRDao.isExitByHql(CoreHqlPack.packDdlSysRoleCheckOnly(role))) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("角色名称已存在")
                    .setErrorSolution("请重新输入角色名称")
                    .build();
        }
    }

    @Override
    public List<SysRole> findAllByBaseBean(SysRole sysRole) {
        Example example = Example.of(sysRole);
        return this.roleRDao.findAll(example);
    }


//    @Override
//    public void packAfterRefreshCache() {
//        if(isCache()){
//            List<SysRole> roles = super.findAll();
//
//            if(CollectionUtils.isNotEmpty(roles)){
//                List<Long> idList = new ArrayList<>();
//                for (SysRole role : roles) {
//                    List<SysRefRoleMenu> refRoleMenuList = findSysRefRoleMenuByRoleId(role.getId());
//
//                    if(CollectionUtils.isNotEmpty(refRoleMenuList)){
//                        idList.clear();
//                        refRoleMenuList.forEach(ref -> idList.add(ref.getMenuId()));
//                        role.setMenuIdList(idList);
//                    }
//                }
//            }
//        }
//    }
}
